#include <SoftwareSerial.h> //Uncomment serial print stuff to tune boundaries !!

// Software serial pins
const int rxPin = -1; // Not used
const int txPin = 3;
SoftwareSerial mySerial(rxPin, txPin);

const int grblPwmInputPin = A1;                 // Connect to GRBL PWM output signal
const int irSamplePin = A2;              // Connect IR detector to this pin
const int spindleEnablePin = 0;
const int irCleanOutputPin = 1; // Connect to RPI to count revolutions (to calc RPM)


int grblPwmInputValue = 0;
int irSamplePinValue = 0;
//boolean spindleEnable      = false;

// set analog voltage value to enable spindle
int spindleEnableThreshold = 10;     // Need to sample your setup to get this number

// Detect when reflection state changes
boolean reflectionDetectedCurrent = false;      // Latest detector reading that was within reflection boundaries
boolean reflectionDetectedPrevious = false;     // Previous detector reading that was within reflection boundaries

//tunable reflection values
const int reflectingBoundaryValue = 200;       //Lower than this value means a reflection is detected
const int noReflectionBoundaryValue = 450;     //Higher than this value means no reflection detected

// the setup routine runs once when you press reset:
void setup() {                
  mySerial.begin(115200);  
  pinMode(spindleEnablePin, OUTPUT);
  pinMode(irCleanOutputPin, OUTPUT); 
}

// the loop routine runs over and over again forever:
void loop() {

  grblPwmInputValue = analogRead(grblPwmInputPin);
  irSamplePinValue = analogRead(irSamplePin);


  // if analog voltage value read from grbl doesnt pass threashold just send 0
  if(grblPwmInputValue > spindleEnableThreshold)
  {
    //spindleEnable = true;
    digitalWrite(spindleEnablePin, HIGH);
    mySerial.println(grblPwmInputValue);
  } else
  {
    //spindleEnable = false;
    digitalWrite(spindleEnablePin, LOW);
    mySerial.println(0);
  }

  // Only SET reflection state if raw value is within boundaries
  if (irSamplePinValue < reflectingBoundaryValue)
  {
    reflectionDetectedCurrent = true;
    
  } else if (irSamplePinValue > noReflectionBoundaryValue)
  {
    reflectionDetectedCurrent = false;
    
  } /*else
  {
    mySerial.write("Just Jitter: ");
    mySerial.println(irSamplePinValue);
  }
    */

  // Only CHANGE state if new state is different than prevouse state
  if (reflectionDetectedCurrent != reflectionDetectedPrevious){
    /*
    if(reflectionDetectedCurrent) //if reflection IS detected
    {
      mySerial.print("sensor = ");
      mySerial.print(irSamplePinValue);
      mySerial.println("\t Reflecting");
    } else
    {
      mySerial.print("sensor = ");
      mySerial.print(irSamplePinValue);
      mySerial.println("\t NO reflection");
    }
    */

    digitalWrite(irCleanOutputPin, reflectionDetectedCurrent); //set pin to HIGH or LOW depending on reflection detection
    reflectionDetectedPrevious = reflectionDetectedCurrent;
  }

//mySerial.print("Test: ");
//mySerial.print(grblPwmInputValue);
//mySerial.print("| ");
//mySerial.println(irSamplePinValue);

  //delay(200);
}
